use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::UPS::API::XML::Builder::Builder;
use Readonly;
use Test::More;
use Data::Dumper;

Readonly my $ROOTPATH => '/TestStore';

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            ## no critic (ProhibitLongLines)
            eval {
                    my $ShippingLabelUPS = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1002/ShippingLabels/1001');## no critic (ProhibitPathConcatenation)
                    isa_ok($ShippingLabelUPS, 'DE_EPAGES::UPS::API::Object::ShippingLabelUPS');
                    my $Builder = DE_EPAGES::UPS::API::XML::Builder::Builder->new($ShippingLabelUPS);
                    my $myAssemble = $Builder->assemble('CreateShippingDocument');
                    is ($myAssemble->{'Request'}->{'RequestOption'},'nonvalidate','subitem Request');
                    note('subitem UPSSecurity');
                    is ($myAssemble->{'UPSSecurity'}->{'UsernameToken'}->{'Username'},'WebserviceUsername',"\t   Username");
                    is ($myAssemble->{'UPSSecurity'}->{'UsernameToken'}->{'Password'},'WebservicePassword',"\t   Password");
                    is ($myAssemble->{'UPSSecurity'}->{'ServiceAccessToken'}->{'AccessLicenseNumber'},'AccessLicenseNumber',"\t   AccessLicenseNumber");
                    note('subitem LabelSpecification');
                    is ($myAssemble->{'LabelSpecification'}->{'LabelImageFormat'}->{'Code'},'GIF',"\t   Code");
                    is ($myAssemble->{'LabelSpecification'}->{'LabelImageFormat'}->{'Description'},'GIF',"\t   Description");
                    is ($myAssemble->{'LabelSpecification'}->{'HTTPUserAgent'},'Mozilla/4.5',"\t   HTTPUserAgent");
                    note('subitem Shipment');
                    is (scalar keys %{$myAssemble->{'Shipment'}},8,'amount of subitems in Shipment');
                 };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestStore' } );
    ImportObjects( 'importOrder.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;