use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::UPS::API::DataCollectors;
use DE_EPAGES::UPS::API::XML::Builder::ShipmentBuilder;
use Readonly;
use Test::More;
use Test::Exception;

Readonly my $ROOTPATH => '/TestStore';
## no critic (ProhibitLongLines)

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                my $ShippingLabelUPS = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1002/ShippingLabels/1001');## no critic (ProhibitPathConcatenation)
                isa_ok($ShippingLabelUPS, 'DE_EPAGES::UPS::API::Object::ShippingLabelUPS');
                my $DataCollectors = DE_EPAGES::UPS::API::DataCollectors->new($ShippingLabelUPS);
                isa_ok($ShippingLabelUPS->get('Order'), 'DE_EPAGES::Order::API::Object::CustomerOrder');
                my $ShipperBuilder = DE_EPAGES::UPS::API::XML::Builder::ShipperBuilder->new($DataCollectors);
                testShipperNumber($ShipperBuilder);
                testAssemble($ShipperBuilder);
            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}
sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestStore' } );
    ImportObjects( 'importOrder.xml', $ROOTPATH );
    return 1;
}

sub testShipperNumber {
    my ($ShipperBuilder) = @_;
    is ($ShipperBuilder->_shipperNumber()->{'ShipperNumber'},'ShipperNumber','call _ShipperNumber with valid Data'); ## no critic (ProhibitPrivateSubroutineUse)
}
sub testAssemble {
    my ($ShipperBuilder) = @_;
    is($ShipperBuilder->assemble()->{'Shipper'}->{'Name'},'ShopFirstName ShopLastName', 'assemble Shipper/Name');
    is($ShipperBuilder->assemble()->{'Shipper'}->{'Address'}->{'AddressLine'},'ShopStreet', 'assemble Shipper/Address/AddressLine');
    is($ShipperBuilder->assemble()->{'Shipper'}->{'TaxIdentificationNumber'},'123456789', 'assemble Shipper/TaxIdentificationNumber');
    is($ShipperBuilder->assemble()->{'Shipper'}->{'ShipperNumber'},'ShipperNumber', 'assemble Shipper/ShipperNumber');
}
RunScript( Sub => \&Main );

1;